import os

import riotwatcher


try:
    token = os.environ['LEAGUE_TOKEN']
except KeyError:
    raise Exception('No LEAGUE_TOKEN found. See developer.riotgames.com')
riot = riotwatcher.RiotWatcher(token)


def get_champ(champ_filter=None):
    champs = []
    for champ in riot.static_get_champion_list()['data'].values():
        champs.append(dict(id=champ['id'], key=champ['key'],
                           name=champ['name']))
        if not champ_filter:
            continue

        if champ_filter == champ['name'].lower():
            return champs[-1:]

    if champ_filter:
        raise Exception('Could not find champion {}', champ_filter)

    return champs



import league_utils.models as models


def test_equality():
    mid = models.Role('Mid')
    assert mid == mid

    top = models.Role('Top')
    assert mid != top

    top_copy = models.Role('Top')
    assert top == top_copy



from riotwatcher import game_maps

import league_utils.api as api


class Item(object):
    def __init__(self, id_, name=None):
        self.id = id_

        self._name = name

        self._builds_from = None
        self._champion = None
        self._champion_unlocked = False
        # TODO: do something with these
        #  ["Butcher's Bridge", 'Howling Abyss', "Summoner's Rift",
        #   'The Crystal Scar', 'The Proving Grounds', 'Twisted Treeline']
        self._maps = None
        # TODO: do something with these
        # ['FlatArmorMod', 'FlatCritChanceMod', 'FlatHPPoolMod',
        #  'FlatHPRegenMod', 'FlatMPPoolMod', 'FlatMPRegenMod',
        #  'FlatMagicDamageMod', 'FlatMovementSpeedMod',
        #  'FlatPhysicalDamageMod', 'FlatSpellBlockMod',
        #  'PercentAttackSpeedMod', 'PercentLifeStealMod',
        #  'PercentMovementSpeedMod']
        self._stats = None
        # TODO: do something else with these?
        # ['Active', 'Armor', 'ArmorPenetration', 'AttackSpeed', 'Aura',
        #  'Bilgewater', 'Boots', 'Consumable', 'CooldownReduction',
        #  'CriticalStrike', 'Damage', 'GoldPer', 'Health', 'HealthRegen',
        #  'Jungle', 'Lane', 'LifeSteal', 'MagicPenetration', 'Mana',
        #  'ManaRegen', 'NonbootsMovement', 'OnHit', 'Slow', 'SpellBlock',
        #  'SpellDamage', 'SpellVamp', 'Stealth', 'Tenacity', 'Trinket',
        #  'Vision']
        self._tags = None

    def __repr__(self):
        return '[{} - {}]'.format(self.id, self.name)

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.id == other.id

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash(self.id)

    # fix missing riot tags
    def _get_missing_tags(self):
        if 'Biscuit of Rejuvenation' in self.name:
            return ['Consumable', 'HealthRegen', 'ManaRegen']
        elif 'Enchantment' in self.name:
            if any(x in self.name for x in ('Alacrity', 'Captain',
                                            'Distortion', 'Furor')):
                return ['Boots']
            elif 'Cinderhulk' in self.name:
                return ['Aura', 'Health', 'Jungle', 'SpellDamage']
            elif 'Devourer' in self.name:
                return ['AttackSpeed', 'Jungle', 'OnHit']
            elif 'Runeglaive' in self.name:
                return ['CooldownReduction', 'Jungle', 'Mana', 'ManaRegen',
                        'OnHit', 'SpellDamage']
            elif 'Runic Echoes' in self.name:
                return ['Jungle', 'NonbootsMovement', 'SpellDamage']
            elif 'Warrior' in self.name:
                return ['CooldownReduction', 'Damage', 'Jungle']
        elif 'Hex Core' in self.name:
            return ['Mana', 'SpellDamage']
        elif 'Fire at Will' in self.name:
            return ['AttackSpeed', 'Damage']
        elif "Death's Daughter" in self.name:
            return ['Damage', 'Slow']
        elif 'Raise Morale' in self.name:
            return ['NonbootsMovement']
        elif 'Poro-Snax' in self.name:
            return []

        raise Exception('No tags for %r.' % self)

    # riot data
    def _load_riot(self):
        info = api.riot.static_get_item(self.id, item_data='all')

        self._name = info['name']

        self._builds_from = [Item(x) for x in info.get('from', list())]

        self._maps = set()
        for map_ in [k for k, v in info['maps'].items() if v]:
            for game_map in [g for g in game_maps if str(g['map_id']) == map_]:
                self._maps.add(game_map['name'])

        self._stats = info['stats']

        try:
            self._tags = set(info['tags']) - set(['Bilgewater'])
        except KeyError:
            self._tags = set(self._get_missing_tags())

        if self.name in ('Boots of Mobility', 'Boots of Swiftness'):
            # Champions that really want movement speed should consider these
            # boots
            self._tags.add('NonbootsMovement')

    @property
    def builds_from(self):
        if isinstance(self._builds_from, type(None)):
            self._load_riot()
        return self._builds_from

    @property
    def maps(self):
        if not self._maps:
            self._load_riot()
        return self._maps

    @property
    def name(self):
        if not self._name:
            self._load_riot()
        return self._name

    @property
    def stats(self):
        if not self._stats:
            self._load_riot()
        return self._stats

    @property
    def tags(self):
        if isinstance(self._tags, type(None)):
            self._load_riot()
        return self._tags

    # additional data
    @property
    def champion(self):
        if not self._champion and not self._champion_unlocked:
            if 'Hex Core' in self.name:
                self._champion = 'Viktor'
            elif any(x in self.name for x in ("Death's Daughter",
                                              'Fire at Will',
                                              'Raise Morale')):
                self._champion = 'Gangplank'
            else:
                self._champion_unlocked = True
        return self._champion



class Role(object):
    def __init__(self, name):
        self.name = name

    def __repr__(self):
        return '[{}]'.format(self.name)

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.name == other.name

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash(self.name)



import pytest

import league_utils.isg as isg
import league_utils.models as models
import league_utils.sources as sources


@pytest.mark.skip(reason='CI needs a real token')
def test_no_duplicate_items():
    champ = models.Champion(133, name='Quinn')
    sources.champion_gg(champ)

    for role in champ.roles:
        set_ = isg.item_set(champ, role)
        items = [item for block in set_[3] for item in block[1]]
        assert len(items) == len(set(items))



import league_utils.output as output


def test_base_dir():
    assert output.get_base_dir('CYGWIN_NT-10.0')
    assert output.get_base_dir('Darwin')
    assert output.get_base_dir('Linux')
    assert output.get_base_dir('Windows')



import league_utils.common as common


def test_dedup():
    assert common.dedup([1, 1, 2]) == [1, 2]
    assert common.dedup(list(range(10)) + list(range(10))) == list(range(10))


def test_without():
    items = [1, 2, 3, 4, 5]

    assert common.without(items, items[-3:]) == items[:-3]

    assert set(common.without(set(items), set(items[-2:]))) == set(items[:-2])

    assert common.without(items, items[-1:], items[:1]) == items[1:-1]



import league_utils.common as common
# TODO: aliasing breaks these lines. WTF?
import league_utils.isg.groups
import league_utils.isg.wants


def item_set(champ, role):
    early = common.without(champ.starts[role],
                           league_utils.isg.groups.consumables(1)) + \
            league_utils.isg.wants.order(
                league_utils.isg.wants.tags(champ, role),
                league_utils.isg.groups.boots())
    build = common.without(champ.builds[role],
                           league_utils.isg.groups.consumables(1), early)

    return (role.name, 'SR', champ.key,
            [('Consumables', league_utils.isg.groups.consumables()),
             ('Early & Boots', common.dedup(early)),
             ('Build', common.dedup(build))])



import league_utils.models as models


# TODO: find similar items
# def all_items():
#     items = []
#     for item in riot.static_get_item_list()['data'].values():
#         items.append(Item(str(item['id']), item['name']))
#     return items


def boots():
    return [
        models.Item('3158', 'Ionian Boots of Lucidity'),
        models.Item('3006', "Berserker's Greaves"),
        models.Item('3009', 'Boots of Swiftness'),
        models.Item('3020', "Sorcerer's Shoes"),
        models.Item('3047', 'Ninja Tabi'),
        models.Item('3111', 'Mercury Treads'),
        models.Item('3117', 'Boots of Mobility'),
    ]


# TODO: complete incomplete items in build
# def completed_items():
#     items = get_all()
#     incomplete = set([i.id for i in functools.reduce(
#         operator.add, [item.builds_from for item in items])])
#     return without([item for item in items if item.id not in incomplete],
#                          get_consumables(1), get_enchantments())


def consumables(extras=False):
    cons = [
        models.Item('2003', 'Health Potion'),
        models.Item('2031', 'Refillable Potion'),
        models.Item('2043', 'Vision Ward'),
        models.Item('3340', 'Warding Trinket'),
        models.Item('3341', 'Sweeping Lens'),
        models.Item('3363', 'Farsight Alteration'),
        models.Item('3364', 'Oracle Alteration'),
    ]
    if extras:
        cons.extend([
            models.Item('2009', 'Total Biscuit of Rejuvenation'),
            models.Item('2010', 'Total Biscuit of Rejuvenation'),
            models.Item('2138', 'Elixir of Iron')])

    return cons


def enchantments():
    return [
        models.Item('1301', 'Enchantment: Alacrity'),
        models.Item('1302', 'Enchantment: Captain'),
        models.Item('1303', 'Enchantment: Distortion'),
        models.Item('1305', 'Enchantment: Furor'),
        models.Item('1306', 'Enchantment: Alacrity'),
        models.Item('1307', 'Enchantment: Captain'),
        models.Item('1308', 'Enchantment: Distortion'),
        models.Item('1317', 'Enchantment: Captain'),
        models.Item('1320', 'Enchantment: Furor'),
        models.Item('1322', 'Enchantment: Captain'),
        models.Item('1323', 'Enchantment: Distortion'),
        models.Item('1328', 'Enchantment: Distortion'),
        models.Item('1330', 'Enchantment: Furor'),
        models.Item('1331', 'Enchantment: Alacrity'),
        models.Item('1409', 'Enchantment: Cinderhulk'),
        models.Item('1413', 'Enchantment: Cinderhulk'),
        models.Item('1414', 'Enchantment: Runic Echoes'),
        models.Item('3241', 'Enchantment: Alacrity'),
        models.Item('3671', 'Enchantment: Warrior'),
        models.Item('3673', 'Enchantment: Runic Echoes'),
        models.Item('3674', 'Enchantment: Devourer'),
    ]



"""usage:
league-utils.py -h | --help
league-utils.py isg [--champ=<champ>] [--write | --no-write]

options:
    isg                    Use the item set generator
    --champ=<champ>        Get info for only this champion.
    --write --no-write     Write to file or print to screen.
    -v --version           Show the version.
    -h --help              Show this screen.
"""
import sys

import docopt

import league_utils.api as api
import league_utils.models as models
import league_utils.isg as isg
# pylint: disable=W0611
import league_utils.isg.encode as encode
import league_utils.output as output
import league_utils.sources as sources


def run():
    args = docopt.docopt(__doc__, version='0.1.9', argv=sys.argv[1:])

    if args['isg']:
        name = None
        if args['--champ']:
            name = args['--champ'].lower()

        champs = api.get_champ(name)
        for c in sorted(champs, key=lambda x: x['name']):
            champ = models.Champion(c['id'], c['key'], c['name'])
            sources.champion_gg(champ)
            for role in champ.roles:
                item_set = isg.item_set(champ, role)
                if args['--write']:
                    # TODO: concurrent:
                    # with concurrent.futures.ProcessPoolExecutor() as exec:
                    #     results = exec.map(main, isg.get_all_champions())
                    # for result in results:
                    #     print('\033[92mok:\033[00m', result.name)
                    data_set = isg.encode.item_set(*item_set)
                    output.to_file(champ, role, data_set)
                    continue

                output.to_screen(champ, role, item_set)

            if args['--write']:
                print('\033[92mok:\033[00m', champ.name)



import os
import platform


def get_base_dir(system):
    if system == 'Windows':
        return os.path.join('C:', 'Riot Games', 'League of Legends')
    elif system == 'Linux':
        return os.path.join(os.path.expanduser('~'), '.PlayOnLinux',
                            'wineprefix', 'LeagueOfLegends', 'drive_c',
                            'Riot Games', 'League of Legends')
    elif system.startswith('CYGWIN_NT'):
        return os.path.join(os.sep, 'cygdrive', 'c', 'Riot Games',
                            'League of Legends')
    elif system == 'Darwin':
        return os.path.join(os.sep, 'Applications', 'League of Legends.app',
                            'Contents', 'LOL')
    else:
        raise Exception('Platform "{}" not supported.'.format(system))


config_dir = os.path.join(get_base_dir(platform.system()), 'Config')
champions_dir = os.path.join(config_dir, 'Champions')


def to_file(champ, role, item_set):
    champion_dir = os.path.join(champions_dir, champ.key, 'Recommended')

    try:
        os.makedirs(champion_dir)
    except OSError:
        pass

    item_set_file = os.path.join(champion_dir,
                                 '{}_{}.json'.format(champ.key, role.name))
    with open(item_set_file, 'w') as f:
        f.write(item_set)


def to_screen(_champ, _role, item_set):
    print('- {} ({})'.format(item_set[0], item_set[1]))
    for block in item_set[3]:
        print('  - {}:'.format(block[0]))
        for item in block[1]:
            print('    - {}'.format(item.name))



import bs4
import requests

import league_utils.models as models


# TODO: merge with other sources
def champion_gg(champ):
    url = 'http://champion.gg/champion/{}'.format(champ.key)
    resp = requests.get(url)
    soup = bs4.BeautifulSoup(resp.text, 'html.parser')

    champ.roles = [models.Role(x.text.strip())
                   for x in soup.select('.champion-profile ul li a h3')]

    for role in champ.roles:
        url = 'http://champion.gg/champion/{}/{}'.format(champ.key, role.name)
        resp = requests.get(url)
        soup = bs4.BeautifulSoup(resp.text, 'html.parser')
        champ.builds[role] = [
            models.Item(x['src'].split('/')[-1].split('.')[0], x['tooltip'])
            for x in soup.select('.col-md-7 .build-wrapper a img')]
        champ.starts[role] = [
            models.Item(x['src'].split('/')[-1].split('.')[0], x['tooltip'])
            for x in soup.select('.col-md-5 .build-wrapper a img')]



import collections
import operator


UNWANTED_TAGS = ['Active', 'Boots', 'Consumable', 'Jungle', 'Lane', 'Trinket']


def order(wants, items):
    ratings = {item: sum(wants[tag] for tag in item.tags) for item in items}
    return [item for item, rating in sorted(ratings.items(),
                                            key=operator.itemgetter(1),
                                            reverse=True) if rating > 0]


def tags(champ, role):
    wanted_tags = collections.Counter()
    for build_tags in [build.tags for build in champ.builds[role]]:
        wanted_tags.update(build_tags - set(UNWANTED_TAGS))

    return wanted_tags



import json


TAG = 'SFT'


def item_set(title, map_, champion, blocks):
    map_ = 'any'
    mode = 'any'
    # TODO: specify mode/map
    # if map_ == 'CS':
    #     mode = 'ODIN'
    # elif map_ == 'HA':
    #     mode = 'ARAM'
    # else:  # 'any', 'SR', or 'TA'
    #     mode = 'CLASSIC'
    #     # TODO: mode = 'any' ?

    return json.dumps({
        'title': '{} {}'.format(TAG, title),
        'map': map_,
        'mode': mode,

        'type': 'custom',
        'sortrank': 1,
        'priority': False,

        'isGlobalForChampions': False,
        'associatedChampions': list(),
        'associatedMaps': [],
        'isGlobalForMaps': True,
        'champion': champion,

        'blocks': [{
            'type': name,
            'items': [{'count': 1, 'id': item.id} for item in items],
        } for name, items in blocks],
    })



from collections import OrderedDict
import functools
import operator


def dedup(l):
    return list(OrderedDict.fromkeys(l))


def without(lh, *rh):
    return [x for x in lh if x not in set(functools.reduce(operator.add, rh))]



#!/usr/bin/env python
import setuptools


setuptools.setup(
    name='league-utils',
    use_scm_version=True,
    description='League of Legends utilities, including an Item Set Generator',
    keywords='league of legends game utilities item set generator isg',
    author='Kevin James',
    author_email='KevinJames@thekev.in',
    url='https://github.com/TheKevJames/league.git',
    license='MIT License',
    packages=setuptools.find_packages(),
    install_requires=['beautifulsoup4', 'docopt', 'requests', 'riotwatcher'],
    setup_requires=['pytest-runner', 'setuptools_scm'],
    tests_require=['pytest', 'pytest-cov', 'pytest-pep8'],
    classifiers=[
        'Programming Language :: Python',
        'Development Status :: 4 - Beta',
        'Natural Language :: English',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
    ],
    entry_points={
        'console_scripts': [
            'league-utils = league_utils.entrypoint:run',
        ],
    },
)



import league_utils.models as models


def test_champion_specific():
    champion_specific = [
        ('Gangplank', "Death's Daughter"),
        ('Gangplank', 'Fire at Will'),
        ('Gangplank', 'Raise Morale'),
        ('Viktor', 'Prototype Hex Core'),
        ('Viktor', 'The Hex Core mk-1'),
        ('Viktor', 'The Hex Core mk-2'),
        ('Viktor', 'Perfect Hex Core'),
    ]

    for c, n in champion_specific:
        assert models.Item(0, name=n).champion == c



import league_utils.sources as sources


class ChampionMock(object):
    def __init__(self, key):
        self.key = key
        self.roles = []
        self.builds = {}
        self.starts = {}


def test_champion_gg():
    key = 'Thresh'

    champ = ChampionMock(key)
    assert champ.key == key
    assert not champ.roles
    assert not champ.builds
    assert not champ.starts

    sources.champion_gg(champ)
    assert champ.key == key
    assert champ.roles
    assert champ.builds
    assert champ.starts



import collections

import league_utils.api as api


class Champion(object):
    def __init__(self, id_, key=None, name=None):
        self.id = id_

        self._key = key  # itemset folder name
        self._name = name  # human-readable
        self._stats = None

        self.roles = list()
        self.builds = collections.defaultdict(list)
        self.starts = collections.defaultdict(list)

    def __repr__(self):
        return '[{} - {}]'.format(self.id, self.name)

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.id == other.id

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash(self.id)

    # riot data
    def _load_riot(self):
        info = api.riot.static_get_champion(self.id, champ_data='all')

        self._key = info['key']
        self._name = info['name']
        self._stats = info['stats']

    @property
    def key(self):
        if not self._key:
            self._load_riot()
        return self._key

    @property
    def name(self):
        if not self._name:
            self._load_riot()
        return self._name

    @property
    def stats(self):
        if not self._stats:
            self._load_riot()
        return self._stats
